// LinkedList.h: interface for the CLinkedList class.
//
//////////////////////////////////////////////////////////////////////
/******************************************************************************
*
*   FILE SPECIFICATION : LinkedList.h
*       TARGET MACHINE : Any machine.
*          ENVIRONMENT : WINDOWS/NT
*               SYSTEM : N/A
*              PROJECT : N/A
*
*   COPYRIGHT
*       Copyright 1995-1999 Ian Duff - Duff Software and Services
*       ALL RIGHTS RESERVED
*
*------------------------------------------------------------------------------
*
*    Author :     Ian Duff
*    Created on : 18 January 1999
*
************* F U N C T I O N A L    D E S C R I P T I O N ********************
*
*   Usage  : #include "LinkedList.h"
*   Purpose: Header file for the CLinkedList class.
*
************* M O D I F I C A T I O N   H I S T O R Y *************************
*     Date       | Author        | Version  | Reason for change
*----------------+---------------+----------+----------------------------------
* 20-March-1999  | I.  Duff      | 2.0W     | Final Realease Windows Version.
*----------------+---------------+----------+---------------------------------
***********************************************************************/
#if !defined(AFX_LINKEDLIST_H__25C278AF_A869_11D2_B8BE_F72E9EEDD90A__INCLUDED_)
#define AFX_LINKEDLIST_H__25C278AF_A869_11D2_B8BE_F72E9EEDD90A__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000
// ---------- Module include files --------------------------------------------
#include <memory.h>						// Required for memcpy()
#include <ctype.h>						// Required for debugging diagnostics
// ---------- Module constant definitions -------------------------------------
static const int Success =  0;				// General Success return code
static const int Fail    = -1;				// General Failure return code

/******************************************************************************
*	Here is the class definition for a list Node class
******************************************************************************/
class CDLitem {
public:
	CDLitem();
	virtual	~CDLitem();
	class	CDLitem *Prev;					// Points to previous node
    class	CDLitem *Next;					// Points to Next node
    LPVOID	Data;							// Points to node data
};

/******************************************************************************
*	Here is the class definition for the List class handler/manager
******************************************************************************/
class CDLList {
public:
	int GetStatus( void );
	virtual ~CDLList();
	CDLList( UINT Size, int ( *pCompData )( const void *, const void*), void( *pFreeData)( void *));

    void    ( *m_pFreedata )( void *);		// Data freeing function
    int     ( *m_pCompdata )( const void *, const void *); // List compare function
	BOOL	m_SortOnInsert;					// Sort when adding to list?
	BOOL	m_Sorted;						// Boolean sorted/unsorted list
	LONG	m_Position;						// Currency position in list
	LONG	m_Count;						// Number of nodes in list
	LONG	m_DataSize;						// Size of data to save
	CDLitem *m_pMark;						// Mark a node in the list
	CDLitem *m_pScanner;					// List "Cursor"
	CDLitem *m_pAnchor;						// List Sentinel

	enum ErrorFlag {						// Possible Errors
		NoErrors = 0,
		InvalidDataSize,					// Bad size
		NoMoreMemory,						// Heap exhausted
		NoComparisonFunction,				// 
		EmptyListReceived,					//
		ListHasNotBeenSorted,				//
		CantDeleteSentinel,					//
		CantMarkSentinel,					//
		NodeNotPreviouslyMarked,			//
		DebugNotDefined,					//
		NullAddress,						//
		InvalidDataLength,					//
		NoCurrency,							//
	};
	ErrorFlag m_eErrorFlag;
};

/******************************************************************************
*	Here is the class definition for the CLinkedList class for MFC Windows.  
*	It has better functionality than the CList class, as it allows for fast 
*	insertion of nodes anywhere in the list.  A node is ALWAYS appended to the 
*	end (or tail) of the list.  Unlike the CList class, it is not necessary to
*	perform sequential searches, as the nodes may be sorted in memory, hence
*	binary searches may be made.
*
*	Technical Note 1.      
*	Apart from the obvious advantages of using linked lists, the user does not 
*	have to worry about specifically removing the list from memory. 
*	The virtual destructor is available, but when the CLinkedList object falls
*	out of scope, all previously allocated memory is automatically released.
*******************************************************************************/
class CLinkedList : public CDLList
{
public:
	BOOL	dlsorted( void );				// BOOL - the list is sorted or not
	int		dlatmark( void );				// Test if at marked node
	int		dltomark( void);				// Go to a marked node in list
	int		dlunmark( void );				// Unmark a node in the list
	int		dlmark( void );					// Mark a Node in list
	int		dlsort( void );					// Sort using primitive swap sort
	int		dlqsort( void );				// Sort using qsort() function
	int		dlreplace( LPVOID data );		// Replace this node with the data
	LPVOID	dlget( void );					// Get currency (the current data)
	int		dldelete( void );				// Delete the current node
	LONG	dlposn( void );					// Node's position in the list
	LONG	dlcount( void );				// Number of nodes in list
	LPVOID	dlbsearch( LPVOID data );		// Binary search of the linked list
	LPVOID	dlfind( LPVOID data );			// Find the node with this data
	int		dlrewind( void );				// Rewind to start of list
	// Change the user supplied comparison function (if required) to another
	int		dlsetcompare( int (*pCompData) (const void *, const void *));
	LPVOID	dlgoback( void );				// Travel backwards through list
	LPVOID	dlgofwd( void );				// Travel forwards through list
	int		dladdins( LPVOID data );		// Add as insertion sort
	int		dladd( LPVOID data );			// Add to the list
	int		dlinsert( LPVOID data );		// Insert a new node here
	LPVOID	dltolast( void );				// Go to the last node in list
	LPVOID	dltofirst( void );				// Go to the first node in list

	// Some operator overloading, use if suited to taste.
	int		operator<<( LPVOID );					// Same as dladd()
	LPVOID	operator[]( LONG );
	// The int argument below enforces postfix increment		
    LPVOID	operator++( int );				// Equates to dlgofwd()	
    LPVOID	operator--( int );				// Equates to dlgoback()

	// Debugging Diagnostic functions
	static int	DumpMemory( LPVOID pAddress = NULL, LONG length = 0, char *pComment = NULL );
	int dldumplist( void );					// Dump the entire list
	int dldumpnode( void );					// Dump a single node

	// The CLinkedList class constructors
	CLinkedList( UINT Size = 0, int ( *pCompData )( const void *, const void*) = NULL, void( *pFreeData)( void *) = NULL);
	// The class destructor
	virtual ~CLinkedList();
	// Internal pointer to the internal add function (dladd or dladdins)
	int ( CLinkedList::*m_pAddFunction )(LPVOID);
	// The internal comparison function used exclusively by the qsort function
	friend	int _Compare(const void *p1, const void *p2 );
	// A temporary static internal CLinkedlist pointer
	static CLinkedList *dllp;

private:
	int dlrestore( void );					// Restore node position numbers
};

#endif // !defined(AFX_LINKEDLIST_H__25C278AF_A869_11D2_B8BE_F72E9EEDD90A__INCLUDED_)
